@@ -73,6 +73,9 @@ def create_fastapi():
                 img = modelify_img.get_input_for_model(preprocess=modelify.model_preprocess, image_source=image_source)
     
                 output = model.run(img)
+                if type(output) is np.ndarray:
+                    output = output.tolist()
+                    
                 return {"result": output}
             except Exception as e:
                 logging.error(e, exc_info=True)
