@@ -1,82 +1,60 @@
 
-import shutil
-import uuid
 import os
-import uuid
+import re
 import requests
 import json
-from modelify.utils.constants import MODELIFY_PRESIGNED_URL, MODELIFY_DEPLOY_URL
+from modelify.utils.constants import APP_FOLDER, MODELIFY_UPLOAD_URL, MODELIFY_DEPLOY_URL, MODELIFY_DOWNLOAD_URL
 from modelify.utils.credential import Credential
 from modelify.utils import message
-import yaml
-from enum import Enum
-from modelify.utils.constants import APP_FOLDER
-from modelify.utils.types import FileTypes
+from modelify.utils.constants import ModelifyFile
+from tqdm import tqdm
 
-""" 
-storage
-  - $USER_ID
-        - $APP_ID
-            - versions
-                - 1
-                    - config.yaml
-                    - model.onnx
-                    - preprocess.pkl
-                    - postprocess.pkl
-                    - requirements.txt
-                    - custom_ui.html
-                - 2
-                    - config.yaml
-                ....
-"""
 
 class ModelifyClient:
     def __init__(self):
         self.credential = Credential()
-
  
     def upload_pipeline(self, upload_types: dict, app_uid, configs):
         # filter upload types
         filetypes = list(map(lambda x: x[0], filter(lambda item: item[1]==True, upload_types.items())))
-        upload_urls = self.get_presigned_url(app_uid, filetypes)
+        upload_urls = self.get_upload_url(app_uid, filetypes)
         for item in upload_urls:
-            self.upload_model_storage(item["url"], item["type"])
+            self.upload_model_storage(item["url"], item["type"], configs)
+        message("Model files uploaded successfully")
         self.register_model(app_uid, configs)
 
-    def get_presigned_url(self ,app_uid, filetypes:list):
+    def get_upload_url(self ,app_uid, filetypes:list):
         data = {'app_uid': app_uid, 'filetypes': filetypes}
         headers = {"api-token": f"{self.credential.api_key}", "Content-Type": "application/json"}
-        req = requests.post(MODELIFY_PRESIGNED_URL, json=data, headers = headers)
+        req = requests.post(MODELIFY_UPLOAD_URL, json=data, headers = headers)
 
         if req.status_code == 200:
             return req.json()["urls"]
         raise Exception("Upload url could not generated")
 
-    def upload_model_storage(self,url, file_type):
-        if file_type == FileTypes.MODEL:
-            blob_name = "model.onnx"
-            content_type = "application/octet-stream"
-        elif file_type == FileTypes.PREPROCESS:
-            blob_name = "preprocess.pkl"
-            content_type = "application/octet-stream"
-        elif file_type == FileTypes.POSTPROCESS:
-            blob_name = "postprocess.pkl"
-            content_type = "application/octet-stream"
-        elif file_type == FileTypes.CONFIG:
-            blob_name = "config.yaml"
-            content_type = "text/yaml"
-        elif file_type == FileTypes.REQUIREMENTS:
-            blob_name = "requirements.txt"
-            content_type = "text/plain"
+    def get_download_urls(self ,app_uid, version=None):
+        data = {'app_uid': app_uid}
+        if version is not None:
+            data["version_number"] = version
+        headers = {"api-token": f"{self.credential.api_key}", "Content-Type": "application/json"}
+        req = requests.post(MODELIFY_DOWNLOAD_URL, json=data, headers = headers)
+
+        if req.status_code == 200:
+            return req.json()["urls"]
+        raise Exception("Download url could not generated")
+
+    def upload_model_storage(self,url, file_type, configs):
+
+        blob_name = ModelifyFile[file_type].file_name
+        content_type = ModelifyFile[file_type].content_type
         
         file_path = os.path.join(APP_FOLDER, blob_name)
         req = requests.put(url, open(file_path, 'rb') , headers= {'Content-Type': content_type})
         
         if req.status_code != 200:
+            print(req.text)
             raise Exception("There is something wrong in model upload stage")
 
-        message("Model uploaded successfully")
-
 
     def register_model(self, app_uid, configs):
         message("Model is registering to your account")
@@ -93,3 +71,13 @@ class ModelifyClient:
                 print("key: ", i, "val: ", response_dict[i])
             print("Something went wrong while model sending.")
 
+    def pull_model(self, app_uid, version=None):   
+        urls = self.get_download_urls(app_uid=app_uid, version=version)
+        for url in urls:
+            response = requests.get(url, stream=True)
+            fname = re.findall("[^\/]+(?=\?[^\/]*$)", url)[0]
+            file_path = os.path.join(APP_FOLDER, fname)
+            with open(file_path, "wb") as handle:
+                for data in tqdm(response.iter_content()):
+                    handle.write(data)
+
